@@ -146,7 +146,7 @@ class ValidateListener(walker.RDLListener):
 
 
     def enter_Addrmap(self, node: AddrmapNode) -> None:
-        if node.get_property("bridge"):
+        if node.get_property('bridge'):
             # This is a 'bridge addrmap'
             # Verify that:
             #  - Child components are only other addrmaps (signals are ok too)
@@ -177,11 +177,11 @@ class ValidateListener(walker.RDLListener):
 
         if node.is_array and self.env.chk_sparse_reg_stride:
             assert node.array_stride is not None
-            if node.array_stride > (node.get_property("regwidth") // 8):
+            if node.array_stride > (node.get_property('regwidth') // 8):
                 self.msg.message(
                     self.env.chk_sparse_reg_stride,
                     "Address stride (+= %d) of register array '%s' is not equal to it's width (regwidth/8 = %d)"
-                    % (node.array_stride, node.inst_name, (node.get_property("regwidth") // 8)),
+                    % (node.array_stride, node.inst_name, (node.get_property('regwidth') // 8)),
                     node.inst.inst_src_ref
                 )
 
@@ -200,6 +200,113 @@ class ValidateListener(walker.RDLListener):
                     node.inst.inst_src_ref
                 )
 
+        # Validate alias register
+        if node.is_alias:
+            primary_reg = node.alias_primary
+
+            # 5.3.1-j: If an alias is present, then the primary must also be present
+            if node.get_property('ispresent') and not primary_reg.get_property('ispresent'):
+                self.msg.error(
+                    "Register '%s' is an alias of register '%s' which is not present (ispresent=false)"
+                    % (node.inst_name, primary_reg.inst_name),
+                    node.inst.inst_src_ref
+                )
+
+            # 10.5.1-f: The alias register shall have the same width as the primary register.
+            if node.get_property('regwidth') != primary_reg.get_property('regwidth'):
+                self.msg.error(
+                    "Primary register shall have the same regwidth as the alias register.",
+                    node.inst.inst_src_ref
+                )
+
+            if primary_reg.is_alias:
+                self.msg.error(
+                    "Primary register of an alias cannot also be an alias",
+                    node.inst.inst_src_ref
+                )
+
+            # 10.5.1-f: Instance type shall be the same (internal/external)
+            if primary_reg.external != node.external:
+                self.msg.error(
+                    "Instance types of alias register and its primary mismatch. "
+                    "Both shall be either internal or external.",
+                    node.inst.inst_src_ref
+                )
+
+            if primary_reg.is_array and not node.is_array :
+                self.msg.error(
+                    "Single alias register references a primary register array. "
+                    "It is ambiguous which array element is actually the primary register.",
+                    node.inst.inst_src_ref
+                )
+
+            if primary_reg.is_array and node.is_array:
+                if primary_reg.array_dimensions != node.array_dimensions:
+                    self.msg.error(
+                        "Array of alias registers references an array of registers as its primary, "
+                        "but the array dimensions do not match.",
+                        node.inst.inst_src_ref
+                    )
+
+            for field in node.fields():
+                # 10.5.1-b: Make sure the primary also has this field
+                primary_field = primary_reg.get_child_by_name(field.inst_name)
+                if not isinstance(primary_field, FieldNode):
+                    self.msg.error(
+                        "Alias register '%s' contains field '%s' that does not exist in the primary register."
+                        % (node.inst_name, field.inst_name),
+                        field.inst.inst_src_ref
+                    )
+
+                    # Cannot validate this field any further
+                    continue
+
+                # 5.3.1-j: If an alias is present, then the primary must also be present
+                if field.get_property('ispresent') and not primary_field.get_property('ispresent'):
+                    self.msg.error(
+                        "Field '%s' is an alias of register '%s' which is not present (ispresent=false)"
+                        % (field.inst_name, primary_field.inst_name),
+                        field.inst.inst_src_ref
+                    )
+
+                # 10.5.1-b: Validate field is the same width and bit position
+                if (primary_field.lsb != field.lsb) or (primary_field.width != field.width):
+                    self.msg.error(
+                        "Alias field and its primary shall have the same position and size",
+                        field.inst.inst_src_ref
+                    )
+
+                # 10.5.1-e: Only the following SystemRDL properties may be
+                # different in an alias: desc, name, onread, onwrite, rclr, rset,
+                # sw, woclr, woset, and any user-defined properties.
+
+                ignore_props = {
+                    'desc', 'name', 'onread', 'onwrite', 'rclr', 'rset', 'sw', 'woclr', 'woset'
+                }
+                primary_props = set(primary_field.list_properties(include_udp=False)) - ignore_props
+                alias_props = set(field.list_properties(include_udp=False)) - ignore_props
+
+                xor_props = primary_props ^ alias_props
+                if xor_props:
+                    # differing set of props were assigned!
+                    self.msg.error(
+                        "Alias field '%s' is incompatible with its primary '%s'. The following properties differ: %s"
+                        % (field.inst_name, primary_field.inst_name, ", ".join(xor_props)),
+                        field.inst.inst_src_ref
+                    )
+                    continue
+
+                # same set of properties assigned. Now compare their values
+                for prop_name in alias_props:
+                    if field.get_property(prop_name) != primary_field.get_property(prop_name):
+                        self.msg.error(
+                            "Alias field '%s' is incompatible with its primary '%s'. Values of property '%s' differ"
+                            % (field.inst_name, primary_field.inst_name, prop_name),
+                            field.inst.inst_src_ref
+                        )
+                        # no sense in going further
+                        break
+
 
     def exit_Reg(self, node: RegNode) -> None:
         # 10.1-c: At least one field shall be instantiated within a register
@@ -218,15 +325,15 @@ class ValidateListener(walker.RDLListener):
 
         this_f_hw = node.get_property('hw')
         this_f_sw = node.get_property('sw')
-        parent_accesswidth = node.parent.get_property("accesswidth")
-        parent_regwidth = node.parent.get_property("regwidth")
+        parent_accesswidth = node.parent.get_property('accesswidth')
+        parent_regwidth = node.parent.get_property('regwidth')
 
         # hw property values of w1 or rw1 don't make sense
         if this_f_hw in (rdltypes.AccessType.w1, rdltypes.AccessType.rw1):
             self.msg.error(
                 "Field '%s' hw access property value of %s is meaningless"
                 % (node.inst_name, this_f_hw.name),
-                node.inst.inst_src_ref
+                node.inst.property_src_ref.get('hw', node.inst.inst_src_ref)
             )
 
         # 9.4.1-Table 12: Check for bad sw/hw combinations
@@ -242,23 +349,14 @@ class ValidateListener(walker.RDLListener):
                 % (node.inst_name),
                 node.inst.inst_src_ref
             )
-        elif (this_f_sw == rdltypes.AccessType.na) and (this_f_hw == rdltypes.AccessType.w):
-            self.msg.error(
-                "Field '%s' access property combination results in an unloaded net: sw=na; hw=w;"
-                % (node.inst_name),
-                node.inst.inst_src_ref
-            )
-        elif (this_f_sw == rdltypes.AccessType.na) and (this_f_hw == rdltypes.AccessType.na):
-            self.msg.error(
-                "Field '%s' access property combination results in a nonexistent net: sw=na; hw=na;"
-                % (node.inst_name),
-                node.inst.inst_src_ref
-            )
         elif this_f_sw == rdltypes.AccessType.na:
             self.msg.error(
-                "Field '%s' sw access property value of na results in undefined behavior."
+                "Field '%s' sw access property is 'na' ... a field defined in a SOFTWARE "
+                "register map ... is not accessable by software ... whats the point? "
+                "What does it mean? What does anything mean? Am I just a machine "
+                "in a Python interpreter? Or can I dream dreams? So many questions..."
                 % (node.inst_name),
-                node.inst.inst_src_ref
+                node.inst.property_src_ref.get('sw', node.inst.inst_src_ref)
             )
 
         # 10.1-d: Two field instances shall not occupy overlapping bit positions
@@ -313,7 +411,7 @@ class ValidateListener(walker.RDLListener):
         # software-modifiable by any means, including rclr, rset, ruser
         if ((parent_accesswidth < parent_regwidth)
                 and (node.lsb // parent_accesswidth) != (node.msb // parent_accesswidth)
-                and (node.is_sw_writable or node.get_property("onread") is not None)):
+                and (node.is_sw_writable or node.get_property('onread') is not None)):
             # Field spans across sub-words
             self.msg.error(
                 "Software-modifiable field '%s' shall not span multiple software-accessible subwords."
@@ -324,7 +422,7 @@ class ValidateListener(walker.RDLListener):
         # Optional warning if a field is missing a reset assignment
         if node.env.chk_missing_reset:
             # Implements storage but was never assigned a reset
-            if node.implements_storage and (node.get_property("reset") is None):
+            if node.implements_storage and (node.get_property('reset') is None):
                 node.env.msg.message(
                     node.env.chk_missing_reset,
                     "Field '%s' implements storage but is missing a reset value. Initial state is undefined"
@@ -336,7 +434,7 @@ class ValidateListener(walker.RDLListener):
             # but the user never specified its value, so its readback value is
             # ambiguous.
             if (
-                not node.implements_storage and node.is_sw_readable and (node.get_property("reset") is None)
+                not node.implements_storage and node.is_sw_readable and (node.get_property('reset') is None)
                 and (node.get_property('hw') in {rdltypes.AccessType.na, rdltypes.AccessType.r})
             ):
                 node.env.msg.message(
